/*-------------------------------------------------------------------
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * %orvr_fitting(model_uid=,type=SEV)
 *
 * Produces reports for all of the Distribution Fitting data.
 * There are 8 Tables that can be created:
 *  1. Selected Severity Distributions for Internal Data
 *  2. Selected Frequency Distributions for Internal Data
 *  3. Selected Severity Distributions for External Data  
 *  4. Selected Severity Distributions for Consortium Data 
 *  5. Selected Frequency Distributions for Consortium Data 
 *
 * Called by %orva_model() - not intended to be called directly!
 *
 * Input:
 *    model_uid - UID for model.
 *    type = SEV or FREQ.
 *
 * Required tables (in project result set area):
 *   varprj.ccmapval_0, 
 *   varprj.ccmapval_1
 *   varprj.cc
 *   varprj.sparmi
 *   varprj.fparmi
 *   varprj.sparmc
 *   varprj.fparmc
 *   varprj.sparmp
 * 
 *-------------------------------------------------------------------*/
%macro orvr_fitting(model_uid=,type=SEV);

   /***- setup some formats we need -***/
   proc format;
      value $KRIReg 'CommonStrKriMForward.txt' = 'Forward'
                    'CommonStrKriMBackward.txt' = 'Backward'
                    'CommonStrKriMMinR' = 'MinR'
                    'CommonStrKriMMaxR' = 'MaxR'
                    other = 'Unknown';
   run;
   proc format;
      value $Sufficiency 'S' = 'Yes'
                         's' = 'Yes'
                         other = 'No';
   run;

   /***- setup our title -***/
   %cleartitles;
   %if &type=SEV or &type=SEVFITDETAILS %then %do;
       title2 &titleSpec2 'Severity Fitting Results Report';
   %end;
   %else %do;
       title2 &titleSpec2 'Frequency Fitting Results Report';
   %end; 
   title3 &titleSpec3 "(Model=&MODEL_NAME Model Author=&FULL_NAME)";
  
  
   /*************** Fitting Options ********************************/
  
   ***** Global Threshold Options;
   title4 &titleSpec4 "Fitting Options - Global Threshold Values";
   proc report data=varprj.MODEL &reportSpec contents='Global Threshold Options'; 
      column 
             ('Fitting' INTERNAL_THRESHOLD EXTERNAL_THRESHOLD CONSORTIUM_THRESHOLD)
             ('Test' INTERNAL_TEST_THRESHOLD EXTERNAL_TEST_THRESHOLD CONSORTIUM_TEST_THRESHOLD )
             ('Default' DEFAULT_INT_THRESHOLD DEFAULT_CON_THRESHOLD DEFAULT_EXT_THRESHOLD )
          ;
      define INTERNAL_THRESHOLD / display 'Int' format= &curfmt RIGHT;
      define EXTERNAL_THRESHOLD  / display 'Pub' format= &curfmt RIGHT;
      define CONSORTIUM_THRESHOLD / display 'Con' format= &curfmt  RIGHT;
      define INTERNAL_TEST_THRESHOLD / display 'Int' format= &curfmt RIGHT;
      define EXTERNAL_TEST_THRESHOLD / display 'Pub' format= &curfmt RIGHT;
      define CONSORTIUM_TEST_THRESHOLD / display 'Con' format= &curfmt RIGHT;
      define DEFAULT_INT_THRESHOLD / display 'Int' format= &curfmt RIGHT;
      define DEFAULT_CON_THRESHOLD  / display 'Pub' format= &curfmt RIGHT;
      define DEFAULT_EXT_THRESHOLD / display 'Con' format= &curfmt RIGHT;
     
   run;

   ***** Global Threshold Usage;
   data usage; set varprj.model;
      USE_INT_FITTING_THRESHOLD = ^USE_INT_DATA_THRESHOLD;
      USE_EXT_FITTING_THRESHOLD = ^USE_EXT_DATA_THRESHOLD;
      USE_CON_FITTING_THRESHOLD = ^USE_CON_DATA_THRESHOLD;
      keep USE_:;
   run;
   title4 &titleSpec4 "Fitting Options - Global Threshold Usage";
   proc report data=usage &reportSpec contents='Global Threshold Usage'; 
      column 
             ('Data' USE_INT_DATA_THRESHOLD USE_EXT_DATA_THRESHOLD USE_CON_DATA_THRESHOLD)
             ('Fitting' USE_INT_FITTING_THRESHOLD USE_EXT_FITTING_THRESHOLD USE_CON_FITTING_THRESHOLD)
             ('Test' USE_INT_TEST_THRESHOLD USE_EXT_TEST_THRESHOLD USE_CON_TEST_THRESHOLD )
          ;
      define USE_INT_DATA_THRESHOLD / display 'Int' format= YesNo.;
      define USE_CON_DATA_THRESHOLD  / display 'Pub' format= YesNo.;
      define USE_EXT_DATA_THRESHOLD / display 'Con' format= YesNo.;
      define USE_INT_FITTING_THRESHOLD / display 'Int' format= YesNo.;
      define USE_CON_FITTING_THRESHOLD  / display 'Pub' format= YesNo.;
      define USE_EXT_FITTING_THRESHOLD / display 'Con' format= YesNo.;
      define USE_INT_TEST_THRESHOLD / display 'Int' format= YesNo.;
      define USE_CON_TEST_THRESHOLD  / display 'Pub' format= YesNo.;
      define USE_EXT_TEST_THRESHOLD / display 'Con' format= YesNo.;      
   run;

   ***** General Fitting Options;
   title4 &titleSpec4 "Fitting Options - General";
   proc report data=varprj.MODEL &reportSpec contents='General Fitting Options'; 
      column FIT_INTERVAL 
              ( 'KRI Regression Method' KRI_REGRESSION_METHOD_SEV KRI_REGRESSION_METHOD_FREQ )
              SUFFICIENCY_CRITERIA          
          ;      
      define FIT_INTERVAL / display 'Fit Interval';
      define KRI_REGRESSION_METHOD_SEV / display 'Severity' format= $KRIReg.;
      define KRI_REGRESSION_METHOD_FREQ / display 'Frequency' format= $KRIReg.;
      define SUFFICIENCY_CRITERIA / display 'Min Sufficiency Criteria';
   run;   
   %if &type=SEV %then %do;
      %let titleText= 'Selected Fittings for Internal Data';
      %SeverityReport(data=varprj.lossi, sparm=varprj.sparmi, graphs=no);
      %let titleText= 'Selected Fittings for Consortium Data';
      %SeverityReport(data=varprj.lossc, sparm=varprj.sparmc, graphs=no);
      %let titleText= 'Selected Fittings for Public Data';
      %SeverityReport(data=varprj.lossp, sparm=varprj.sparmp, graphs=no);
   %end;
   %else %if &type=SEVFITDETAILS %then %do;
      %let titleText= 'Selected Fittings for Internal Data';
      %SeverityReport(data=varprj.lossi, sparm=varprj.sparmi, graphs=yes);
      %let titleText= 'Selected Fittings for Consortium Data';
      %SeverityReport(data=varprj.lossc, sparm=varprj.sparmc, graphs=yes);
      %let titleText= 'Selected Fittings for Public Data';
      %SeverityReport(data=varprj.lossp, sparm=varprj.sparmp, graphs=yes);
   %end;
   %else %do;
     %let titleText= 'Selected Fittings for Internal Data';
     data _IntwithCount;
        merge varprj.sparmi(keep=ccuid obsCount)
              varprj.fparmi(drop=number_of_observations);
        by ccuid;
     run;
     %FrequencyReport(data=_IntwithCount);
     %let titleText= 'Selected Fittings for Consortium Data';
     data _ConwithCount;
        merge varprj.sparmc(keep=ccuid obsCount)
              varprj.fparmc(drop=number_of_observations);
        by ccuid;
     run;
     %FrequencyReport(data=_ConwithCount);
   %end;
  
%mend orvr_fitting;

%macro SeverityReport(data=, sparm=, graphs=);
   title4 &titleSpec4 &titleText;  
   %global nobs;
   %fit_checkEmpty(data=&sparm, selected=_selected);
   %if &nobs ^= 0 %then %do;  
      *** get Names for Cross Classification Variables;
      proc sql;
      create table _work as
         select distinct * 
         from _selected f,
              varprj.cc cc,
              varprj.ccmapval_0 ccv0,
              varprj.ccmapval_1 ccv1
         where f.CCUID = cc.CCUID and
               cc.CCVUID_0 = ccv0.CCVUID_0 and 
               cc.CCVUID_1 = ccv1.CCVUID_1 ; 
      quit;
      ods graphics on;

      /* Set up a temporary file to write our data step code to */
      data _NULL_;
         length path $256;
         path = pathname("work");
         put path=;
         call symput( "path", trim(path) );
      run;

      filename tmp "&path/build_data_sas_tmp";
      data _NULL_; set _work;
         file tmp;
         put 'Title5 "' CCVVALUE_0 ' and ' CCVVALUE_1'";';
         put '%severity_details_one_cell( data=' "&data, sparm=&sparm, graphs=&graphs, ccuid=" ccuid ");";
      run;

      /* run the program we just made */
      %inc "&path/build_data_sas_tmp";
      ods graphics off;    
   %end;
    
%mend SeverityReport;

%macro FrequencyReport(data=);
   title4 &titleSpec4 &titleText;  
   %global nobs;
   %fit_checkEmpty(data=&data,selected=_selected,freq=true);
   %if &nobs ^= 0 %then %do;  
      data _NULL_; set _selected;
         array fparms FParam_:;
         num_fparms = dim(fparms);
         call symput('NUM_FPARAMS',num_fparms);
      run;

      *** get Names for Cross Classification Variables;
      proc sql;
      create table _work as
         select distinct * 
         from _selected f,
              varprj.cc cc,
              varprj.ccmapval_0 ccv0,
              varprj.ccmapval_1 ccv1
         where f.CCUID = cc.CCUID and
               cc.CCVUID_0 = ccv0.CCVUID_0 and 
               cc.CCVUID_1 = ccv1.CCVUID_1 ; 
      quit;
     
      /*- fix up cc strings -*/
      proc sort data=_work; by CCUID; run;
      data _work; set _work;
         retain foo;
         length foo $ 128;
         if trim(left(foo)) ne "" and trim(left(foo)) eq trim(left(CCVVALUE_0)) then do;
            foo = CCVVALUE_0;
            CCVVALUE_0="";
         end;
         else do; 
            foo=CCVVALUE_0; 
         end;
      run;
   
      *** print report;
      proc report data=_work &reportSpec contents="Frequency Fittings";
         column ( 'Classifications' CCVVALUE_0 CCVVALUE_1) 
                ( 'Fitted Statistics' obsCount MEAN VARIANCE )
                ( 'Fitting Parameters' DISTRIBUTION 
                %do i=1 %to &NUM_FPARAMS; FPARAM_&i %end;
                )
                ( 'Goodness of Fit Statistics' LOGL SBC AIC );
         define CCVVALUE_0/ left "&MAPNAME0";
         define CCVVALUE_1/ display left "&MAPNAME1";
         define obsCount/ display format=COMMA10.0 'Count';
         define MEAN/ display format=&deffmt 'Mean' RIGHT;
         define VARIANCE/ display  format=&deffmt 'Variance' RIGHT;
         define DISTRIBUTION/ display 'Dist';
         %do i=1 %to &NUM_FPARAMS;
            define FPARAM_&i/ display format=&deffmt "FParm&i";
         %end;
         define LOGL/ display format=&deffmt 'LogL';
         define SBC/ display format=&deffmt 'SBC';
         define AIC/ display format=&deffmt 'AIC';
      run;
   %end;  
%mend FrequencyReport;

%macro fit_checkEmpty(data=,selected=,freq=false);
   %let nobs=0;
   %let dsid=%sysfunc(open(&data,I));
   %if &dsid ^= 0 %then %do;
      %let nobs=%sysfunc(attrn(&dsid,nlobs));
      %let rc=%sysfunc(close(&dsid));
   %end;
   %if &nobs = 0 %then %do;
      %fit_emptyReport(No data found);
   %end;  
   %else %do;
      data &selected; set &data; 
         where SELECTION='S'
            %if &freq eq true %then %do;
            and SEVITER_SELECTED=1
            %end;
         ; 
      run;
      %let nobs=0;
      %let dsid=%sysfunc(open(&selected,I));
      %if &dsid ^= 0 %then %do;
         %let nobs=%sysfunc(attrn(&dsid,nlobs));
         %let rc=%sysfunc(close(&dsid));
      %end;
      %if &nobs = 0 %then %do;
         %fit_emptyReport(No data found);
      %end; 
   %end; 
%mend fit_checkEmpty;

%macro fit_emptyReport(phrase);
   data fake;                                    
      length fakevar $10; label fakevar = '00'X;
      fakevar= ' ';
   run;
   proc report &reportSpec data=fake;   
      column fakevar;
      define fakevar / display "&phrase";
   run;
%mend fit_emptyReport;

%macro severity_details_one_cell( 
      data=,             /* internal, consortium, or public data set */
      sparm=,
      ccuid=,
      graphs=yes );

   data sparm; set &sparm; where ccuid=&ccuid and selection='S';
      call symput("dist", trim(distribution));
      call symput("ks_v",ks_v);
      call symput("cvm_v",cvm_v); 
      call symput("ad_v",ad_v);
      call symput("aic",aic);
      call symput("sbc",sbc);
      call symput("logl",logl);
      call symput("obscount",obscount);
      call symput("mean",mean);
      call symput("std",std);
      if kurtosis =. then kurtosis=1.0e10;  /* Can't handle missing for now*/
      call symput("kurtosis",kurtosis);
   run;
   %orvd_&dist;

   %if &graphs=yes %then %do;
      %orva_DiagPlot(data=&data, ccuid=&ccuid, dist=&dist, maxpoints=500, diagdata = dd, parmdata=sparm );
      proc summary data=dd;
         var amount;
         output out=max max=max min=min;
      run;

      /* Set up the points for the histogram --------------*/
      data _null_;set max; 
         call symput("max_amount", max);
         call symput("max_log",log(max)-log(min)+.5);
         call symput("histticks", trim(0)||" "||trim( round(max/2,10000))||" "||trim(round(max,10000)));
      run;

      /* modify frequency numbers to match histogram scale */
      data dd; set dd;
         log_amount = log(amount);
         freq = freq * &max_amount / 30;
      run;

      /* Set up the template ---------------------------*/
      proc template;
        define statgraph orvar.Histogram;
          dynamic ks_v cvm_v ad_v aic sbc logl OBSCOUNT MEAN STD KURTOSIS;
          layout gridded;
            layout lattice / rows=2 columns=3;
               sidebar / align=top;
                  layout gridded/columns=1 valign=top;
                     entrytitle "Diagnostic Plots and Statistics for Distribution &dist_name";
                  endlayout;
               endsidebar;
               Layout Overlay / xaxisopts=(label="CDF Difference" labelfontsize=11 ticks=(-1 0 1)) 
                  yaxisopts=(label="Model CDF" labelfontsize=11);
                  scatterplot y=tcdf x=cdfdif/ markersize=2px
                              markersymbol=GraphDataDefault:markersymbol
                              markercolor=GraphDataDefault:contrastcolor;
                              lineparm xintercept=0/
                              linecolor=GraphReferenceLines:foreground
                              extreme=true;
               EndLayout;
               Layout Overlay / xaxisopts=(label="Model Probability" labelfontsize=11) 
                  yaxisopts=(label="Empirical Probability" labelfontsize=11);
                  scatterplot y=ecdf x=tcdf/ markersize=2px
                              markersymbol=GraphDataDefault:markersymbol
                              markercolor=GraphDataDefault:contrastcolor;
                              lineparm yintercept=0 slope=1/
                              linecolor=GraphReferenceLines:foreground
                              linepattern=StatGraphFitLine:linestyle
                              linethickness=StatGraphFitLine:linethickness
                              extreme=true;
               EndLayout;
               Layout Overlay / xaxisopts=(label="Net Loss" labelfontsize=11 ) 
                  yaxisopts=(label="Cumulative Probability" labelfontsize=11);
                  scatterplot y=ecdf x=amount/ markersize=2px
                              markersymbol=GraphDataDefault:markersymbol
                              markercolor=GraphDataDefault:contrastcolor;
                  scatterplot y=tcdf x=amount/ markersize=2px
                              markercolor=GraphReferenceLines:foreground
                              markersymbol=StatGraphFitLine:linestyle;
               EndLayout;
               Layout Overlay / xaxisopts=(label="Model Quantile" labelfontsize=11) 
                  yaxisopts=(label="Empirical Quantile" labelfontsize=11);
                  scatterplot y=quant x=amount/ markersize=2px
                              markersymbol=GraphDataDefault:markersymbol
                              markercolor=GraphDataDefault:contrastcolor;
                  scatterplot y=amount x=amount/ markersize=2px
                              markercolor=GraphReferenceLines:foreground
                              markersymbol=StatGraphFitLine:linestyle;
               EndLayout;
               Layout Overlay / xaxisopts=(label="Severity Distribution" labelfontsize=11 ticks=(&histticks)) 
                  yaxisopts=(label="Number of Events" labelfontsize=11);
                  Histogram amount / nbins=30 scale=Proportion fillcolor=GraphDataDefault:foreground;
                  scatterplot y=freq x=amount/ markersize=2px
                              markercolor=GraphReferenceLines:foreground
                              markersymbol=StatGraphFitLine:linestyle;
               EndLayout;
               Layout Overlay / xaxisopts=(label="Log Severity" labelfontsize=11) 
                  yaxisopts=(label="Number of Events" labelfontsize=11);
                  Histogram log_amount / nbins=30 scale=Proportion fillcolor=GraphDataDefault:foreground;
                  scatterplot y=eval(freq * &max_log / &max_amount * amount ) x=log_amount/ markersize=2px
                              markercolor=GraphReferenceLines:foreground
                              markersymbol=StatGraphFitLine:linestyle;
               EndLayout;
            EndLayout;
            Layout Overlay;
               layout gridded /  columns=8 valign=center
                  background=GraphWalls:background border=on;
                  entry "NObs"   / halign=left  valign=top;
                  entry  obscount / halign=right valign=top;
                  entry "Mean"  / halign=left  valign=top;
                  entry mean       / halign=right valign=top;
                  entry "Std"  / halign=left  valign=top;
                  entry std       / halign=right valign=top;
                  entry "Kurtosis"  / halign=left  valign=top;
                  entry kurtosis       / halign=right valign=top;
               endlayout;
            EndLayout;
            Layout lattice / rows=1 columns=3;
               Layout Overlay;
                  layout gridded /  columns=2 valign=center
                     background=GraphWalls:background border=on;
                     entry "Kolmogorov-Smirnov"   / halign=left  valign=top;
                     entry  ks_v     / halign=right valign=top;
                     entry "Cramer-von Mises"  / halign=left  valign=top;
                     entry cvm_v       / halign=right valign=top;
                  endlayout;
               EndLayout;
               Layout Overlay;
                  layout gridded /  columns=2 valign=center
                     background=GraphWalls:background border=on;
                     entry "Anderson-Darling" / halign=left  valign=top;
                     entry ad_v       / format=8.2 halign=right valign=top;
                     entry "Log-Likelihood" / halign=left  valign=top;
                     entry  logl / format=8.2 halign=right valign=top;
                  endlayout;
               EndLayout;
               Layout Overlay;
                  layout gridded /  columns=2 valign=center
                     background=GraphWalls:background border=on;
                     entry "AIC" / halign=left  valign=top;
                     entry aic / format=8.2 halign=right valign=top;
                     entry "SBC" / halign=left  valign=top;
                     entry  sbc / format=8.2 halign=right valign=top;
                  endlayout;
               EndLayout;
            EndLayout;
          endlayout;
        end;
      run;

      /* Output the graph ------------------------------*/
      data _NULL_;set dd;   
         file print ods=(template='orvar.Histogram' 
              dynamic=(ks_v=&ks_v cvm_v=&cvm_v ad_v=&ad_v
                       aic=&aic sbc=&sbc logl=&logl OBSCOUNT=&obscount 
                       MEAN=&mean STD=&std KURTOSIS=&kurtosis) );
         put _ods_;
      run;
   %end;  /* end if graph=yes */

   /* Print parameter and Kri parameter tables */
   %ORVR_sfit_stats(sparm=sparm, fileref="", title="", ClearTitles=FALSE);

%mend;

